const system = client.registerSystem(0, 0);
var thisPlayer = {};
var thisDialog = { player: {}, title: "", content: "" };
var thisCpuData = {};
var isUIOpen = 0;
var defaultOptions = {
	always_accepts_input: true,
	render_game_behind: true,
	absorbs_input: true,
	is_showing_menu: true,
	should_steal_mouse: false,
	force_render_below: true,
	render_only_when_topmost: false
};

system.initialize = function () {
	this.listenForEvent("minecraft:client_entered_world", (eventData) => this.onStartSetup(eventData));
	this.listenForEvent("minecraft:ui_event", (eventData) => this.onUIMessage(eventData));
	this.listenForEvent("cpu:openCpuUI_toClient", (eventData) => this.onOpenCpuUI(eventData));
	this.listenForEvent("cpu:closeCpuUI_toClient", (eventData) => this.onCloseCpuUI(eventData));
	this.listenForEvent("cpu:fillStorageUI_toClient", (eventData) => this.onfillStorageUI(eventData));
	this.listenForEvent("cpu:updateRedSignal_toClient", (eventData) => this.onUpdRedSignal(eventData));
	this.listenForEvent("cpu:showDialog_toClient", (eventData) => this.onShowDialog(eventData));

	this.registerEventData("cpu:cpu_update_fromClient", { player: {}, dataToSave: "" });
};

system.onStartSetup = function (eventData) {
	thisPlayer = eventData.data.player;
};

system.update = function () {
};

system.shutdown = function () {
	// this.unloadUI('dialog.html');
	// this.unloadUI("cpu.html");
};

system.loadUI = function (path) {
	let loadUiEvent = this.createEventData('minecraft:load_ui');
	loadUiEvent.data = { path: path, options: defaultOptions };
	this.broadcastEvent('minecraft:load_ui', loadUiEvent);
};

system.unloadUI = function (path) {
	let unloadUIEvent = this.createEventData('minecraft:unload_ui');
	unloadUIEvent.data = { path: path };
	this.broadcastEvent('minecraft:unload_ui', unloadUIEvent);
};

system.onUIMessage = function (eventData) {
	if (eventData.data === "cpu:closeUI_fromUI") {
		this.closeUI();
	}

	if (eventData.data === "cpu:closeDialog_fromUI") {
		this.unloadUI('dialog.html');
		if (isUIOpen == -1) {
			isUIOpen = 1;
			let uiEventData = this.createEventData("minecraft:send_ui_event");
			uiEventData.data.eventIdentifier = "cpu:toggleUI_toUI";
			this.broadcastEvent("minecraft:send_ui_event", uiEventData);
		}
	}

	if (eventData.data === "cpu:cpuUILoaded_fromUI") {
		let uiEventData = this.createEventData("minecraft:send_ui_event");
		uiEventData.data.eventIdentifier = "cpu:fillCpuUI_toUI";
		uiEventData.data.data = JSON.stringify(thisCpuData);
		this.broadcastEvent("minecraft:send_ui_event", uiEventData);
	}

	if (eventData.data === "cpu:dialogLoaded_fromUI") {
		let uiEventData = this.createEventData("minecraft:send_ui_event");
		uiEventData.data.eventIdentifier = "cpu:fillDialog_toUI";
		uiEventData.data.data = JSON.stringify(thisDialog);
		this.broadcastEvent("minecraft:send_ui_event", uiEventData);
	}

	if (eventData.data.match(/^cpu:saveData_fromUI:/)) {
		let updateEvent = this.createEventData("cpu:cpu_update_fromClient");
		updateEvent.data = JSON.parse(eventData.data.substr(20));
		updateEvent.data.player = thisPlayer;
		this.broadcastEvent("cpu:cpu_update_fromClient", updateEvent);
	}
};

system.onfillStorageUI = function (eventData) {
	let uiEventData = this.createEventData("minecraft:send_ui_event");
	uiEventData.data.eventIdentifier = "cpu:fillStorageUI_toUI";
	uiEventData.data.data = JSON.stringify(eventData.data);
	this.broadcastEvent("minecraft:send_ui_event", uiEventData);
}

system.onOpenCpuUI = function (eventData) {
	if (eventData.data.player.id != thisPlayer.id) return;
	isUIOpen = 1;
	this.loadUI("cpu.html");
	thisCpuData = eventData.data;
};

system.onCloseCpuUI = function (eventData) {
	if (eventData.data.nameId != thisCpuData.nameId) return;
	this.closeUI();
};

system.onUpdRedSignal = function (eventData) {
	if (eventData.data.nameId != thisCpuData.nameId) return;

	let uiEventData = this.createEventData("minecraft:send_ui_event");
	uiEventData.data.eventIdentifier = "cpu:updateRedSignal_toUI";
	uiEventData.data.data = JSON.stringify(eventData.data);
	this.broadcastEvent("minecraft:send_ui_event", uiEventData);
};

system.onShowDialog = function (eventData) {
	if (isUIOpen == 1) {
		isUIOpen = -1;
		let uiEventData = this.createEventData("minecraft:send_ui_event");
		uiEventData.data.eventIdentifier = "cpu:toggleUI_toUI";
		this.broadcastEvent("minecraft:send_ui_event", uiEventData);
	}
	if (eventData.data.player.id != thisPlayer.id) return;
	this.loadUI('dialog.html');
	thisDialog = eventData.data;
};

system.closeUI = function () {
	this.unloadUI("cpu.html");
	isUIOpen = 0;
};

{// Helper functions
	system.say = function (msg) {
		let eventData = this.createEventData("minecraft:display_chat_event");
		eventData.data.message = JSON.stringify(msg);
		this.broadcastEvent("minecraft:display_chat_event", eventData);
	};
}