let thisOSK = {};
let thisPlayer = {};
let currentPage = 1;
let totalPages = 1;
let oldNameId = "";
let killMeStatus = 0;

const cpuUI = document.getElementById("cpuUI");
const editorUI = document.getElementById("editorUI");
const nameId = document.getElementById("nameId");
const storageDiv = document.getElementById("storageDiv");
const slotList = document.getElementById("slotList");
const redSignal = document.getElementById("redSignal");
const salida = document.getElementById("salida");
const killMeBtn = document.getElementById("killMe");
const abortKillMeBtn = document.getElementById("abortKillMe");
const currentPageText = document.getElementById("currentPage");
const totalPagesText = document.getElementById("totalPages");
const title = document.getElementById("title");
const editArea0 = document.getElementById("editArea0");
const editArea1 = document.getElementById("editArea1");
const emptyNameAlert = document.getElementById("emptyNameAlert");

editorUI.style.display = "none";

let scriptInterface = null;
engine.on("facet:updated:core.scripting", function (interface) {
	scriptInterface = interface;
});
engine.trigger("facet:request", ["core.scripting"]);
scriptInterface.triggerEvent("cpu:cpuUILoaded_fromUI");

engine.on("cpu:fillCpuUI_toUI", function (eventData) {
	const data = JSON.parse(eventData);
	thisPlayer = data.player;
	oldNameId = nameId.value = data.nameId;
	fillStorageUI(data.storage);
	redSignal.textContent = data.redSignal;
});

engine.on("cpu:fillStorageUI_toUI", function (eventData) {
	const data = JSON.parse(eventData);
	oldNameId = nameId.value = data.nameId;
	fillStorageUI(data.storage);
});

engine.on("cpu:updateRedSignal_toUI", function (eventData) {
	const data = JSON.parse(eventData);
	redSignal.textContent = data.redSignal;
});

engine.on("cpu:toggleUI_toUI", function () {
	toggleUI();
});

// document.body.addEventListener('keypress', e => {
// 	e.preventDefault();
// 	e.stopImmediatePropagation();
// 	e.stopPropagation();
// });

document.body.addEventListener('keydown', e => {
	if (cpuUI.style.display == "none" && editorUI.style.display == "none") return;
	switch (e.keyCode) {
		case 27:
			closeUI();
			break;
		case 13:
			if (editorUI.style.display == "block") {
				saveEditor();
			}
			break;
		case 9:
			if (thisOSK !== {} && title.textContent == "Edit Slot") {
				const currentEditArea = (thisOSK.inputElement.id == "editArea0") ? "editArea1" : "editArea0";
				focusInput(currentEditArea);
				thisOSK.setInput(currentEditArea);
			}
			if (currentEditArea == "editArea1" || currentEditArea == "editArea0")
				e.preventDefault();
			break;
	}
});

slotDataExec = function (id) {
	const thisId = id.replace(/^btn_exec_/, "");
	const slotId = document.getElementById(`id_${thisId}`).value;
	const dataToSave = JSON.stringify({ uType: "slotExec", nameId: oldNameId, slotId: slotId });
	scriptInterface.triggerEvent("cpu:saveData_fromUI:" + dataToSave);
	report(`Exec "${slotId}"`);
}

slotDataRem = function (id) {
	const thisId = id.replace(/^btn_rem_/, "");
	const slotId = document.getElementById(`id_${thisId}`).value;
	const dataToSave = JSON.stringify({ uType: "slotRem", nameId: oldNameId, slotId: slotId });
	scriptInterface.triggerEvent("cpu:saveData_fromUI:" + dataToSave);
	report(`Remove "${slotId}"`);
}

toggleUI = function () {
	cpuUI.style.display = cpuUI.style.display == "none" ? "block" : "none";
};

fillStorageUI = function (storage) {
	//50 = storageDiv.firstElementChild.offsetHeight;
	const pageSize = Math.floor(storageDiv.offsetHeight / 50);
	storageDiv.innerHTML = "";
	if (storage == null) storage = {};
	let itemCount = 0;
	let thisPage = 1;
	for (let slotId in storage) {
		if (slotId == "_scope_")
			continue;
		itemCount++;
		if (itemCount > pageSize) {
			itemCount = 1;
			thisPage++;
		}

		const slotData = storage[slotId].slotData;
		const div = document.createElement("DIV");
		div.className = "slotLine page_" + thisPage;
		div.innerHTML = `
			<textarea id="id_${slotId}" class="slotId" onclick="editSlot('${slotId}', 0)">${slotId}</textarea>
			<textarea id="slot_${slotId}" class="slotData" onclick="editSlot('${slotId}', 1)">${slotData}</textarea>
			<span id="btn_exec_${slotId}" class="btn Exec" onclick="slotDataExec(this.id)"></span>
			<span id="btn_rem_${slotId}" class="btn Rem" onclick="slotDataRem(this.id)"></span>
		`;
		storageDiv.appendChild(div);
	}
	totalPages = thisPage;
	showSlots();
}

closeUI = function () {
	if (editorUI.style.display == "block") {
		closeEditor();
	} else {
		scriptInterface.triggerEvent("cpu:closeUI_fromUI");
	}
};

pageUp = function () {
	currentPage--;
	if (currentPage < 1) currentPage = 1;
	showSlots();
}

pageDown = function () {
	currentPage++;
	if (currentPage > totalPages) currentPage = totalPages;
	showSlots();
}

showSlots = function () {
	totalPagesText.textContent = totalPages;
	currentPageText.textContent = currentPage;
	const slots = document.getElementsByClassName("slotLine");
	for (let i = 0; i < slots.length; i++) {
		slots[i].style.display = "none";
	}
	const shownSlots = document.getElementsByClassName(`page_${currentPage}`);
	for (let i = 0; i < shownSlots.length; i++) {
		shownSlots[i].style.display = "inline";
	}
}

report = function (msg) {
	salida.value = `${now()} ${msg}\n` + salida.value;
}

now = function () {
	const now = new Date();
	const h = now.getHours().toString().padStart(2, "0");
	const m = now.getMinutes().toString().padStart(2, "0");
	const s = now.getSeconds().toString().padStart(2, "0");
	return h + ":" + m + ":" + s + " -";
}

editNameId = function () {
	cpuUI.style.display = "none";
	editorUI.style.display = "block";

	title.textContent = "Edit Name";
	editArea0.value = nameId.value;
	editArea1.style.display = "none";

	thisOSK = new onScreenKeyboard("OSK", "editArea0");
	focusInput("editArea0");
}

editSlot = function (slotId, activeInputIndex) {
	cpuUI.style.display = "none";
	editorUI.style.display = "block";

	title.textContent = "Edit Slot";
	editArea0.value = slotId != "" ? document.getElementById(`id_${slotId}`).value : "";
	editArea1.value = slotId != "" ? document.getElementById(`slot_${slotId}`).value : "";
	editArea1.style.display = "";

	const activeInputId = (activeInputIndex == 0) ? "editArea0" : "editArea1";
	thisOSK = new onScreenKeyboard("OSK", activeInputId);
	focusInput(activeInputId);
}

focusInput = function (elementId) {
	const thisElement = document.getElementById(elementId);
	thisElement.focus();
	thisElement.selectionStart = thisElement.selectionEnd = thisElement.value.length;
}

saveEditor = function () {
	let dataToSave = "";
	let errStatus = false;
	switch (true) {
		case (editArea0.value == ""):
			errStatus = " <--- Name field cannot be empty.";
			break;
		case (editArea0.value.length > 6):
			errStatus = " <--- Names must be 6 characters long max.";
			break;
		case (/[^a-zA-Z0-9_\-]/.test(editArea0.value)):
			errStatus = " <--- Valid characters for names are letters, numbers and underscores.";
			break;
	}
	if (errStatus) {
		focusInput("editArea0");
		emptyNameAlert.innerHTML = errStatus;
		window.setTimeout(function () {
			emptyNameAlert.innerHTML = "";
		}, 2500);
		return;
	}
	switch (title.textContent) {
		case "Edit Slot":
			dataToSave = JSON.stringify({ uType: "slotChange", nameId: oldNameId, slotId: editArea0.value, slotData: editArea1.value });
			report(`Slot "${editArea0.value}" "${editArea1.value}"`);
			break;
		case "Edit Name":
			if (oldNameId == editArea0.value) break;
			dataToSave = JSON.stringify({ uType: "nameId", nameId: oldNameId, newNameId: editArea0.value });
			report(`CPU Name changed to "${editArea0.value}"`);
			break;
	}
	scriptInterface.triggerEvent("cpu:saveData_fromUI:" + dataToSave);
	closeEditor();
}

closeEditor = function () {
	editorUI.style.display = "none";
	cpuUI.style.display = "block";
};

killMe = function () {
	switch (killMeStatus) {
		case 0:
			killMeStatus = 1;
			// killMeBtn.className = "killMe1";
			killMeBtn.innerHTML = "Sure?";
			abortKillMeBtn.className = "";
			break;
		case 1:
			killMeStatus = 2;
			// killMeBtn.className = "killMe2";
			killMeBtn.innerHTML = "Go!";
			break;
		case 2:
			dataToSave = JSON.stringify({ uType: "killMe", nameId: oldNameId });
			scriptInterface.triggerEvent("cpu:saveData_fromUI:" + dataToSave);
			break;
	}
	killMeBtn.className = "killMe" + killMeStatus;
};

abortKillMe = function () {
	killMeStatus = 0;
	killMeBtn.className = "killMe0";
	killMeBtn.innerHTML = "Kill me";
	abortKillMeBtn.className = "hideAbort";
}