class onScreenKeyboard {
	constructor(containerDivId, inputElementId) {
		this.containerDiv = document.getElementById(containerDivId);
		this.shift = this.caps = false;
		this.setInput(inputElementId);
		this.display();
		this.recentTouch = false;
	}

	display() {
		let self = this;
		let keys = [
			[
				[1.5, "Esc", "esc"],
				[1, "1", 49, "!", 33],
				[1, "2", 50, "@", 64],
				[1, "3", 51, "#", 35],
				[1, "4", 52, "$", 36],
				[1, "5", 53, "%", 37],
				[1, "6", 54, "^", 94],
				[1, "7", 55, "&", 38],
				[1, "8", 56, "*", 42],
				[1, "9", 57, "(", 40],
				[1, "0", 48, ")", 41],
				[1, "-", 45, "_", 95],
				[1, "=", 61, "+", 43],
				[2.5, "Bksp", "bksp"]
			],
			[
				[1, "~", 126, "§", 167],
				[1, "q", 113, "Q", 81],
				[1, "w", 119, "W", 87],
				[1, "e", 101, "E", 69],
				[1, "r", 114, "R", 82],
				[1, "t", 116, "T", 84],
				[1, "y", 121, "Y", 89],
				[1, "u", 117, "U", 85],
				[1, "i", 105, "I", 73],
				[1, "o", 111, "O", 79],
				[1, "p", 112, "P", 80],
				[1, "[", 91, "{", 123],
				[1, "]", 93, "}", 125],
				[1, "\\", 92, "|", 124]
			],
			[
				[2.5, "Caps", "caps"],
				[1, "a", 97, "A", 65],
				[1, "s", 115, "S", 83],
				[1, "d", 100, "D", 68],
				[1, "f", 102, "F", 70],
				[1, "g", 103, "G", 71],
				[1, "h", 104, "H", 72],
				[1, "j", 106, "J", 74],
				[1, "k", 107, "K", 75],
				[1, "l", 108, "L", 76],
				[1, ";", 59, ":", 58],
				[1, "'", 39, '"', 34],
				[3.5, "Save", "save"]
				// [3.5, "Del", "del"]
			],
			[
				[3.5, "Shift", "shift"],
				[1, "z", 122, "Z", 90],
				[1, "x", 120, "X", 88],
				[1, "c", 99, "C", 67],
				[1, "v", 118, "V", 86],
				[1, "b", 98, "B", 66],
				[1, "n", 110, "N", 78],
				[1, "m", 109, "M", 77],
				[1, ",", 44, "<", 60],
				[1, ".", 46, ">", 62],
				[1, "/", 47, "?", 63],
				[4, "Shift", "shift"]
			],
			[
				[3, "Home", "home"],
				[2, "<-", "left"],
				[10, "Spacebar", 32],
				[2, "->", "right"],
				[3, "End", "end"]
			]
		];

		this.containerDiv.innerHTML = "";
		this.containerDiv.className = "onScreenKeyboard";

		for (let a = 0; a < keys.length; a++) {
			let thisRow = document.createElement("span");
			thisRow.className = "row";
			for (let b = 0; b < keys[a].length; b++) {
				for (let c = 1; c < keys[a][b].length; c += 2) {
					let thisKey = keys[a][b];
					let thisKeyButton = document.createElement("span");
					thisKeyButton.className = "key";
					if (c == 3) {
						thisKeyButton.className += " alt"
						thisKeyButton.style.display = "none";
					} else if (thisKey.length == 5) {
						thisKeyButton.className += " base"
					};
					thisKeyButton.style.width = (thisKey[0] + 1.3) + "em";
					thisKeyButton.innerHTML = thisKey[c];
					if (thisKey[c + 1] == "save") {
						thisKeyButton.onclick = function () { saveEditor() };
					} else if (thisKey[c + 1] == "esc") {
						thisKeyButton.onclick = function () { closeEditor() };
					} else {
						thisKeyButton.ontouchstart = thisKeyButton.onmousedown = function (e) {
							if (self.recentTouch) return;
							if (e.type == "touchstart") self.setRecentTouch();
							self.sendKey(thisKey[c + 1]);
							self.intervalId = setInterval(function () {
								self.sendKey(thisKey[c + 1]);
							}, 150);
						};
						thisKeyButton.ontouchend = thisKeyButton.onmouseup = function (e) {
							clearInterval(self.intervalId);
							self.inputElement.focus();
							self.setRecentTouch();
						};
					}
					thisRow.appendChild(thisKeyButton);
				}
			}
			this.containerDiv.appendChild(thisRow);
		}
	}

	setRecentTouch() {
		let self = this;
		self.recentTouch = true;
		setInterval(function () {
			self.recentTouch = false;
		}, 75);
	}

	sendKey(keyCode) {
		let input = this.inputElement;
		let start = input.selectionStart;
		let end = input.selectionEnd;
		let textLength = 0;

		switch (keyCode) {
			case "caps":
				this.toggleCaps();
				break;
			case "shift":
				this.toggleShift();
				break;
			case "bksp":
				if (start > 0) {
					input.value = input.value.slice(0, start - 1) + input.value.slice(end, input.value.length);
					textLength = -1;
				}
				break;
			// case "del":
			//   if (end < input.value.length) {
			//     input.value = input.value.slice(0, start) + input.value.slice(end + 1, input.value.length);
			//     textLength = -1;
			//   }
			//   break;
			case "left":
				textLength = input.selectionStart == 0 ? 0 : - 1;
				break;
			case "right":
				textLength = 1;
				break;
			case "home":
				start = 0;
				break;
			case "end":
				start = input.value.length;
				break;
			case "save":
				break;
			default:
				textLength = 1;
				if ((input.value.length > 5 && input.id == "editArea0") || (input.value.length > 16383 && input.id == "editArea1") ) break;
				input.value = input.value.substring(0, start)
					+ String.fromCharCode(keyCode)
					+ input.value.substring(end, input.value.length);
				if (this.shift) this.toggleShift();
				break;
		}

		input.selectionStart = input.selectionEnd = start + textLength;
		this.inputElement.focus();
	}

	toggleAlts() {
		let baseKeys = document.getElementsByClassName("base");
		let altKeys = document.getElementsByClassName("alt");
		let altsOn = (this.shift || this.caps) && !(this.shift && this.caps);
		for (let i = 0; i < baseKeys.length; i++) {
			baseKeys[i].style.display = altsOn ? "none" : "";
			altKeys[i].style.display = altsOn ? "" : "none";
		}
	}

	toggleShift() {
		this.shift = !this.shift;
		this.toggleAlts();
	}

	toggleCaps() {
		this.caps = !this.caps;
		this.toggleAlts();
	}

	setInput(inputElementId) {
		this.inputElement = document.getElementById(inputElementId);
	}
}